"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = useDebounce;

var _react = require("react");

/**
 * useDebounce
 * Creates a stable-identity debounced function.
 * The wrapped function has two methods attached:
 *    `.cancel()` to cancel any pending invocations
 *    `.flush()` to immediately execute an pending invocations
 * The wrapped function returns a Promise that will be resolved with the return value of the debounced function once its execution is completed.
 * @param fn the function to be debounced
 * @param ms duration in milliseconds since the last invocation that must elapse before the next invocation is executed
 */
function useDebounce(fn, ms) {
  var callback = (0, _react.useRef)(fn);
  var args = (0, _react.useRef)([]);
  var context = (0, _react.useRef)();
  var returnValueResolve = (0, _react.useRef)(function () {});
  var timeoutId = (0, _react.useRef)();
  (0, _react.useEffect)(function () {
    callback.current = fn;
  }, [fn]);
  return (0, _react.useMemo)(function () {
    var execute = function execute() {
      timeoutId.current = undefined;
      var resolve = returnValueResolve.current;
      resolve(callback.current.apply(context.current, args.current));
    };

    var cancel = function cancel() {
      if (timeoutId.current) {
        window.clearTimeout(timeoutId.current);
        timeoutId.current = undefined;
        return true;
      }

      return false;
    };

    var flush = function flush() {
      if (cancel()) {
        execute();
      }
    };

    function debouncedFn() {
      cancel();

      for (var _len = arguments.length, _args = new Array(_len), _key = 0; _key < _len; _key++) {
        _args[_key] = arguments[_key];
      }

      args.current = _args;
      context.current = this;
      timeoutId.current = window.setTimeout(execute, ms);
      return new Promise(function (resolve) {
        returnValueResolve.current = resolve;
      });
    }

    return Object.assign(debouncedFn, {
      cancel: cancel,
      flush: flush
    });
  }, [ms]);
}