/**
 * There are situations where a React component needs to know the hover state
 * of a child/sibling component, most commonly for applying complementary hover styles
 *
 * Usage Example:
 *
 * const Foo = () => {
 *  const { isHovered, ref } = useHover<HTMLButtonElement>();
 *  return (
 *    <>
 *      <button ref={ref}>Click me</button>
 *      <div>{isHovered ? 'button hovered' : 'button not hovered'}</div>
 *    </>
 *  )
 * }
 */

import * as React from 'react';

interface IUseHoverReturnType<T extends HTMLElement> {
  isHovered: boolean;
  ref: React.MutableRefObject<T | null>;
}

export default <T extends HTMLElement>(): IUseHoverReturnType<T> => {
  const [isHovered, setIsHovered] = React.useState(false);
  const ref = React.useRef<T | null>(null);

  const onMouseOverHandler = () => {
    setIsHovered(true);
  };
  const onMouseOutHandler = () => {
    setIsHovered(false);
  };

  React.useEffect(() => {
    const element = ref?.current;

    if (ref && element) {
      element.addEventListener('mouseenter', onMouseOverHandler);
      element.addEventListener('mouseleave', onMouseOutHandler);

      return () => {
        element.removeEventListener('mouseenter', onMouseOverHandler);
        element.removeEventListener('mouseleave', onMouseOutHandler);
      };
    }
  });

  return { isHovered, ref };
};
